-- Project: GridConfigurableLayouts r96
-- File: LDBQuickSelect.lua
-- Last Modified: 2012-04-10T08:35:25Z
-- Author: msaint
-- Project desc: Allows in-game creation of highly customized layouts in Grid
-- File desc: Allows quick selection of layout from Grid LDB icon using button4 

local FILE_VERSION = tonumber("93") or 5000

--{{{ Get or Create the module
local gcl = Grid:GetModule("GridConfigurableLayouts", true) or Grid:NewModule("GridConfigurableLayouts", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
gcl.Versions = gcl.Versions or {} 
if (gcl.Versions.LDBQuickSelect and gcl.Versions.LDBQuickSelect >= FILE_VERSION) then return end
gcl.Versions.LDBQuickSelect = FILE_VERSION
---}}}


--{{{ Libraries and modules
--local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
local gridLDB = (LibStub:GetLibrary("LibDataBroker-1.1")):GetDataObjectByName("Grid")
local GridRoster = Grid:GetModule("GridRoster")
local GridLayout = Grid:GetModule("GridLayout")
--}}}


--{{{ Hack the localization function for easy implementation later
gcl.L = gcl.L or {} 
local L = gcl.L 
setmetatable(L,{ __index = function(l, text) return text end})
---}}}


---{{{ Make everthing local that we can
   --Lua library:
local pairs = pairs 
   --WoW api:

---}}}


--{{{ make debug function local for convenience
local function debug(...)
   gcl:Debug("[GCL-LDB]", ...)
end
--}}}


--{{{ Local variables and objects
local menu = CreateFrame("Frame", "GCLDropDownFrame", UIParent, "UIDropDownMenuTemplate")
local gclDB = {}

local function menuOnLoad(f, level)
-- Loading function for both levels of the left click menu that is added on the
-- Grid LDB icon.
   local info
   local layoutNames = {}
   -- For level one, show GCL layouts, for level two, show all other layouts.
   local layoutSource = ((level == 1) and gclDB.layouts) or GridLayout.layoutSettings
   for name, _ in pairs(layoutSource) do --Make a list of the names we need
      if level == 1 or (not gclDB.layouts[name]) then
         table.insert(layoutNames, name)
      end
   end
   if next(layoutNames) then
      table.sort(layoutNames)
   end
   if level == 1 then
      -- Offer the functionality we replaced, i.e. toggle Grid frame lock
      local profile = GridLayout.db.profile 
      info = UIDropDownMenu_CreateInfo()
      info.text = (profile.FrameLock and L["Unlock Grid Frame"]) or L["Lock Grid Frame"]
      info.value = "LockUnlockFrame"
      info.notCheckable = true
      info.func = function()
   			GridLayout.db.profile.FrameLock = not GridLayout.db.profile.FrameLock
   			LibStub("AceConfigRegistry-3.0"):NotifyChange("Grid")
   			GridLayout:UpdateTabVisibility()
         end
      UIDropDownMenu_AddButton(info, 1)
      -- Add an entry to reach sub-menu with generic layouts
      info = UIDropDownMenu_CreateInfo()
      info.text = L["Other Layouts"]
      info.value = "otherLayoutsSubmenu"
      info.hasArrow = true
      info.notCheckable = true
      UIDropDownMenu_AddButton(info, 1)
      -- Add descriptive entry for gcl layouts
      info = UIDropDownMenu_CreateInfo()
      info.text = L["GCL Layouts"]
      info.value = "GCLLayouts"
      info.isTitle = true
      info.notCheckable = true
      UIDropDownMenu_AddButton(info, 1)
   end
   for _, name in ipairs(layoutNames) do --Add entries for all of our layouts     
      info = UIDropDownMenu_CreateInfo() 
      info.text = name
      info.value = name
      info.checked = GridLayout.db.profile.layouts[GridRoster:GetPartyState()] == name 
      info.arg1 = name
      info.func = function(self, layoutName) -- layoutName comes from arg1 above
            local partyState = GridRoster:GetPartyState() 
            if GridLayout.db.profile.layouts[partyState] ~= layoutName then 
               GridLayout.db.profile.layouts[partyState] = layoutName
               GridLayout:ReloadLayout()
               --UIErrorsFrame:AddMessage(string.format(L["Layout for %q changed to %q"], partyState, layoutName), 1, 0, 0)
               RaidNotice_AddMessage(RaidWarningFrame, string.format(L["Layout for %q changed to %q"], partyState, layoutName), ChatTypeInfo["RAID_WARNING"])
            end  
         end
      info.arg1 = name
      UIDropDownMenu_AddButton(info, level)
   end
end


-- Hook the existing click and tooltip handlers on the Grid icon.
-- These must be hooked manually because of peculiarities in the way acehook
-- and LDB work.
local oldOnClick = gridLDB["OnClick"]
gridLDB["OnClick"] = function (self, button)
   if (button == "LeftButton") and gclDB.ldbQuickSelect and not InCombatLockdown() then
      GameTooltip:Hide()
      ToggleDropDownMenu(1, nil, menu, self, 0, 0)
   else
      oldOnClick(self, button)
	end
end

local oldOnTooltipShow = gridLDB["OnTooltipShow"]
gridLDB["OnTooltipShow"] = function(tooltip)
	tooltip:AddLine("Grid", 1, 1, 1)
	if gclDB.ldbQuickSelect then
      if InCombatLockdown() then 
      	tooltip:AddLine(L["Click to change layouts."], 0.5, 0.5, 0.5)
      	tooltip:AddLine(L["Right-Click for more options."], 0.5, 0.5, 0.5)
		else
			tooltip:AddLine(L["Click to change layouts."])
			tooltip:AddLine(L["Right-Click for more options."])
		end
	else
	   oldOnTooltipShow(tooltip)
	end
end

Grid.options.args.quickselect = {
	order = -2,
	name = L["Use minimap icon to change layouts."],
	desc = L["Right clicking the Grid minimap or LDB icon brings up a layout quickselect menu but does not toggle Grid frame lock."],
	width = "double",
	type = "toggle",
  	get = function () return gclDB.ldbQuickSelect end,
  	set = function () gclDB.ldbQuickSelect = not gclDB.ldbQuickSelect end,
}

-- gcl.options.args.globalOptions.args["ldbQuickSelect"] = { 
--    type = "toggle",
--   	name = L["Right click on Grid icon to quickly select layouts (instead of toggling frame lock)."],
--   	desc = L["Right clicking the Grid icon brings up a layout quickselect menu but does not toggle Grid frame lock."],
--   	width = "full",
--    order = 5,
--   	disabled = InCombatLockdown,
--   	get = function () return gclDB.ldbQuickSelect end,
--   	set = function () gclDB.ldbQuickSelect = not gclDB.ldbQuickSelect end,
-- }

function gcl:initLDBQuickSelect()
   gclDB = gcl.gcldb
   UIDropDownMenu_Initialize(menu, menuOnLoad, "MENU")
end
